/*******************************************************************************************************************************************************
 * Copyright  2016 <WIZnet Co.,Ltd.> 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the Software), 
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************************************************************************************************/
/**
  ******************************************************************************
  * @file    WZTOE/Loopback/main.c 
  * @author  IOP Team
  * @version V1.0.1
  * @date    19-May-2015
  * @brief   Main program body
  ******************************************************************************
  * @attention
  * @par Revision history
  *    <2015/05/19> V1.0.1 by justinKim
  *      1. Print Address %.3d -> %d 
  *    <2015/05/01> 1st Release
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, WIZnet SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2015 WIZnet Co.,Ltd.</center></h2>
  ******************************************************************************
  */ 
/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>

#include "W7500x_gpio.h"
#include "W7500x_uart.h"
#include "W7500x_crg.h"
#include "W7500x_adc.h"
#include "W7500x_wztoe.h"
#include "W7500x_miim.h"

#include "wizchip_conf.h"
#include "socket.h"

/* Private typedef -----------------------------------------------------------*/
UART_InitTypeDef UART_InitStructure;

/* Private define ------------------------------------------------------------*/
#define __DEF_USED_MDIO__ 
#define __DEF_USED_IC101AG__ //for W7500 Test main Board V001

/* Private function prototypes -----------------------------------------------*/
void delay(__IO uint32_t milliseconds); //Notice: used ioLibray
void TimingDelay_Decrement(void);

/* Private variables ---------------------------------------------------------*/
static __IO uint32_t TimingDelay;


///////////////////////////////////////
// Debugging Message Printout enable //
///////////////////////////////////////
#define _MAIN_DEBUG_

///////////////////////////
// Demo Firmware Version //
///////////////////////////
#define VER_H		1
#define VER_L		00

//////////////////////////////////////////////////
// Socket & Port number definition for Examples //
//////////////////////////////////////////////////
#define SOCK_TCPS       0
#define SOCK_UDPS       1
#define PORT_TCPS       5000
#define PORT_UDPS       3000

///////////////////////////
// Shared Buffer         //
///////////////////////////
#define DATA_BUF_SIZE 2048

uint8_t gDATABUF[DATA_BUF_SIZE] = {0x00, };
uint16_t gDATABUF_size = 0;

///////////////////////////
// Network Configuration //
///////////////////////////
static void Net_Conf(void);
static void Display_Net_Conf(void);

wiz_NetInfo gWIZNETINFO = { {0x00, 0x08, 0xdc, 0xab, 0xcd, 0xff},	// MAC
                            {192, 168, 11, 114},					// IP
                            {255, 255, 255, 0},						// Subnet
                            {192, 168, 11, 1},						// Gateway
                            {0, 0, 0, 0},							// DNS
                            NETINFO_STATIC };						// DHCP enable

////////////////////////////////////////////////////////////////////////////////////
// W5500 Ethernet Compliance Test: Defines / Variables / Functions                //
////////////////////////////////////////////////////////////////////////////////////
#define _TEST_DEBUG_

#define MACRAW_SOCKNUM	0
#define UDP_SOCKNUM		1

#define WIZCHIP_ETHERNET_MTU_SIZE	1514
#define WIZCHIP_UDP_MTU_SIZE		1472

#define TEST_STATE_MACRAW_BYTE		0
#define TEST_STATE_MACRAW_RANDOM	1
#define TEST_STATE_UDP_BYTE			2
#define TEST_STATE_UDP_RANDOM		3
#define TEST_STATE_IDLE				4

#define TEST_BYTE					0xff

uint8_t test_state;

uint8_t destip[4] = {255, 255, 255, 255}; // broadcast IP
uint16_t destport = 55000;

void ComplianceTest_Data_Preparation(uint8_t state);
uint16_t ComplianceTest_Send_Data(uint8_t type, uint8_t sn);
uint16_t fill_buffer_random_data(uint8_t * buf, uint16_t datasize);
uint16_t fill_buffer_user_data(uint8_t * buf, uint16_t datasize, uint8_t databyte);

// Switch Status
uint8_t SW_STATUS0 = 0;
uint8_t SW_STATUS1 = 0;

///////////////////////////
// ADC                   //
///////////////////////////
int ADC_Read(ADC_CH num);

////////////////////////////////////////
// User's Application running Control //
////////////////////////////////////////
uint8_t run_user_applications = 0;

/**
  * @brief   Main program
  * @param  None
  * @retval None
  */
int main()
{
#ifdef _MAIN_DEBUG_
	uint8_t tmpstr[6] = {0,};
#endif

    /* External Clock */
    CRG_PLL_InputFrequencySelect(CRG_OCLK);

    /* Set Systme init */
    SystemInit();
		
		// WIZ750SR Watchdog IC Setting
		*(volatile uint32_t *)(0x41001170) = 0x0002; // 8MHz output (125n)
		*(volatile uint32_t *)(0x41002008) = 0x0002; // PAD PA_02 use as CLKOUT
	
    /* UART1 Init */
    UART_StructInit(&UART_InitStructure);
    UART_Init(UART1,&UART_InitStructure);
		
		/*UART2 Init*/
		S_UART_Init(115200);

    /* SysTick_Config */
    SysTick_Config((GetSystemClock()/1000));

    /* Set WZ_100US Register */
    setTIC100US((GetSystemClock()/10000));
    //getTIC100US();	
    //printf(" GetSystemClock: %X, getTIC100US: %X, (%X) \r\n", 
    //      GetSystemClock, getTIC100US(), *(uint32_t *)TIC100US);        
		
		// SW initialize
		GPIO_Configuration(GPIOC, GPIO_Pin_12, GPIO_Mode_IN, PAD_AF1);
		GPIO_Configuration(GPIOC, GPIO_Pin_13, GPIO_Mode_IN, PAD_AF1);
		
#ifdef __DEF_USED_IC101AG__ //For using IC+101AG
    *(volatile uint32_t *)(0x41003068) = 0x64; //TXD0 - set PAD strengh and pull-up
    *(volatile uint32_t *)(0x4100306C) = 0x64; //TXD1 - set PAD strengh and pull-up
    *(volatile uint32_t *)(0x41003070) = 0x64; //TXD2 - set PAD strengh and pull-up
    *(volatile uint32_t *)(0x41003074) = 0x64; //TXD3 - set PAD strengh and pull-up
    *(volatile uint32_t *)(0x41003050) = 0x64; //TXE  - set PAD strengh and pull-up
#endif	
    printf("PHY is linked. \r\n"); 
#ifdef __DEF_USED_MDIO__ r
    /* mdio Init */
    mdio_init(GPIOB, MDC, MDIO );
		
		set_link(FullDuplex10);
    /* PHY Link Check via gpio mdio */
    while( link() == 0x0 )
    {
        printf(".");  
        delay(500);
    }
    printf("PHY is linked. \r\n");  
#else
    delay(1000);
    delay(1000);
#endif

	/* Network Configuration */
	Net_Conf();
	
	/* Flag for running user's code  */
	run_user_applications = 1;
	
#ifdef _MAIN_DEBUG_
	ctlwizchip(CW_GET_ID,(void*)tmpstr);
	
  printf("\r\n==============================================\r\n");
	printf(" WIZnet %s Ethernet Compliance Test v%d.%.2d\r\n", tmpstr, VER_H, VER_L);
	printf("==============================================\r\n");
	printf(" >> State #0: MACRAW Send mode, 0x%.2X\r\n", TEST_BYTE);
	printf(" >> State #1: MACRAW Send mode, Random data\r\n");
	printf(" >> State #2: UDP Broadcast Send mode, 0x%.2X\r\n", TEST_BYTE);
	printf(" >> State #3: UDP Broadcast Send mode, Random data\r\n");
	printf(" >> State #4: IDLE State\r\n");
	printf("==============================================\r\n");

	Display_Net_Conf(); // Print out the network information to serial terminal
#endif

	/* Main loop ***************************************/

	////////////////////////////////////////////////////////////////////////////////////
	// W5500 Ethernet Compliance Test: Settings
	////////////////////////////////////////////////////////////////////////////////////
	// initial test mode
	#define __TEST_MODE__
	// __TEST_MODE__ is output random.
	//#define __RANDOM_MODE__
	// Switch Control, Macrow data random -> Byte mode change
	//#define __SW_TEST__
	////////////////////////////////////////////////////////////////////////////////////

  // Switch Control
	
#ifdef __TEST_MODE__
	//test_state = TEST_STATE_MACRAW_BYTE;
	test_state = TEST_STATE_MACRAW_RANDOM;
#endif

#ifdef _TEST_DEBUG_
	printf(" > Ethernet Compliance Test Start\r\n\r\n");
	printf(" >> State #%d\r\n", test_state);
#endif

#ifdef __RANDOM_MODE__
	ADC_Init();
#endif

	while(1)
	{
		// TODO: insert user's code here
		if(run_user_applications)
		{
#ifdef __SW_TEST__
			SW_STATUS0 = GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_13);
			SW_STATUS1 = GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_12);
			
			if(SW_STATUS0 == 0)
			{
					test_state = TEST_STATE_MACRAW_BYTE;
					printf(" >> State MACRAW\r\n");
			}
			else if(SW_STATUS1 == 0)
			{
					test_state = TEST_STATE_MACRAW_RANDOM;
					printf(" >> State # RANDOM \r\n");
			}
#endif
			ComplianceTest_Data_Preparation(test_state);
			
			switch(test_state)
			{
				case TEST_STATE_MACRAW_BYTE:
				case TEST_STATE_MACRAW_RANDOM:
					ComplianceTest_Send_Data(SOCK_MACRAW, MACRAW_SOCKNUM);
					break;

				case TEST_STATE_UDP_BYTE:
				case TEST_STATE_UDP_RANDOM:
					ComplianceTest_Send_Data(SOCK_UDP, UDP_SOCKNUM);
					break;

				case TEST_STATE_IDLE:
				default:
					break;
			}
		} // End of user's code
	}
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in milliseconds.
  * @retval None
  */
void delay(__IO uint32_t milliseconds)
{
  TimingDelay = milliseconds;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

static void Net_Conf(void)
{
	/* wizchip netconf */
	ctlnetwork(CN_SET_NETINFO, (void*) &gWIZNETINFO);
}

static void Display_Net_Conf(void)
{
#ifdef _MAIN_DEBUG_
	uint8_t tmpstr[6] = {0,};
#endif

	ctlnetwork(CN_GET_NETINFO, (void*) &gWIZNETINFO);

#ifdef _MAIN_DEBUG_
	// Display Network Information
	ctlwizchip(CW_GET_ID,(void*)tmpstr);

	if(gWIZNETINFO.dhcp == NETINFO_DHCP) printf("\r\n===== %s NET CONF : DHCP =====\r\n",(char*)tmpstr);
		else printf("\r\n===== %s NET CONF : Static =====\r\n",(char*)tmpstr);
	printf(" MAC : %02X:%02X:%02X:%02X:%02X:%02X\r\n", gWIZNETINFO.mac[0], gWIZNETINFO.mac[1], gWIZNETINFO.mac[2], gWIZNETINFO.mac[3], gWIZNETINFO.mac[4], gWIZNETINFO.mac[5]);
	printf(" IP : %d.%d.%d.%d\r\n", gWIZNETINFO.ip[0], gWIZNETINFO.ip[1], gWIZNETINFO.ip[2], gWIZNETINFO.ip[3]);
	printf(" GW : %d.%d.%d.%d\r\n", gWIZNETINFO.gw[0], gWIZNETINFO.gw[1], gWIZNETINFO.gw[2], gWIZNETINFO.gw[3]);
	printf(" SN : %d.%d.%d.%d\r\n", gWIZNETINFO.sn[0], gWIZNETINFO.sn[1], gWIZNETINFO.sn[2], gWIZNETINFO.sn[3]);
	printf("=======================================\r\n");
#endif
}

uint16_t fill_buffer_random_data(uint8_t * buf, uint16_t datasize)
{
	uint16_t i;
	uint16_t seed = 0;

	if(datasize > DATA_BUF_SIZE) datasize = DATA_BUF_SIZE;

	seed = ADC_Read(ADC_CH0);
	//printf("ADC_Read = %d\r\n", seed);
	
	srand(seed);

	for(i = 0; i < datasize; i++)
	{
		buf[i] = (uint8_t)(rand() % 0xff);
	}

	/*
	printf("\r\n");
	for(i = 0; i < datasize; i++) printf("%.2X ", buf[i]);
	printf("\r\n\r\n");
	*/

	return datasize;
}

uint16_t fill_buffer_user_data(uint8_t * buf, uint16_t datasize, uint8_t databyte)
{
	uint16_t i;

	if(datasize > DATA_BUF_SIZE) datasize = DATA_BUF_SIZE;

	for(i = 0; i < datasize; i++)
	{
		buf[i] = databyte;
	}

	/*
	printf("\r\n");
	for(i = 0; i < datasize; i++) printf("%.2X ", buf[i]);
	printf("\r\n\r\n");
	*/

	return datasize;
}

void ComplianceTest_Data_Preparation(uint8_t state)
{
	uint8_t i = 0;
	static uint8_t prev_state = 0;
	static uint8_t flag_data_generated = 0;

	if(state != prev_state)
	{
		prev_state = state;
		flag_data_generated = 0;
	}

	switch(state)
	{
		case TEST_STATE_MACRAW_BYTE:
			//if(!flag_data_generated)
			//{
				gDATABUF_size = fill_buffer_user_data(gDATABUF, WIZCHIP_ETHERNET_MTU_SIZE, TEST_BYTE);
				//flag_data_generated = 1;
			//}
			break;

		case TEST_STATE_MACRAW_RANDOM:
			gDATABUF_size = fill_buffer_random_data(gDATABUF, WIZCHIP_ETHERNET_MTU_SIZE);
			break;

		case TEST_STATE_UDP_BYTE:
			if(!flag_data_generated)
			{
				gDATABUF_size = fill_buffer_user_data(gDATABUF, WIZCHIP_UDP_MTU_SIZE, TEST_BYTE);
				flag_data_generated = 1;
			}
			break;

		case TEST_STATE_UDP_RANDOM:
			gDATABUF_size = fill_buffer_random_data(gDATABUF, WIZCHIP_UDP_MTU_SIZE);
			break;

		case TEST_STATE_IDLE:
		default:
			break;
	}

	if((state == TEST_STATE_MACRAW_BYTE) || (state == TEST_STATE_MACRAW_RANDOM))
	{
		/////////////////////////////////////////////////////////////////////////////////
		// MACRAW Packet Settings
		/////////////////////////////////////////////////////////////////////////////////
		// Destination MAC address; Broadcast MAC address
		gDATABUF[i++] = 0xff;
		gDATABUF[i++] = 0xff;
		gDATABUF[i++] = 0xff;
		gDATABUF[i++] = 0xff;
		gDATABUF[i++] = 0xff;
		gDATABUF[i++] = 0xff;
		// Source MAC address; Device MAC address
		gDATABUF[i++] = gWIZNETINFO.mac[0];
		gDATABUF[i++] = gWIZNETINFO.mac[1];
		gDATABUF[i++] = gWIZNETINFO.mac[2];
		gDATABUF[i++] = gWIZNETINFO.mac[3];
		gDATABUF[i++] = gWIZNETINFO.mac[4];
		gDATABUF[i++] = gWIZNETINFO.mac[5];
		// Protocol Type
		gDATABUF[i++] = 0x08;
		gDATABUF[i++] = 0x00;
		/////////////////////////////////////////////////////////////////////////////////
	}
}

uint16_t ComplianceTest_Send_Data(uint8_t type, uint8_t sn)
{
	uint16_t len = 0;
	uint16_t dummyport = 0;
	//int i=0;

	switch(getSn_SR(sn))
	{
		case SOCK_MACRAW :
		case SOCK_UDP :
			if(gDATABUF_size > DATA_BUF_SIZE) len = DATA_BUF_SIZE;
			len = sendto(sn,  gDATABUF, gDATABUF_size, destip, destport);
			//printf("%X", len);
			//delay(1000);
			//for(i = 0; i < gDATABUF_size; i++) printf("%.2X ", gDATABUF[i]);
			//delay(2000);
			break;

		case SOCK_CLOSED:
			switch(type)
			{
				case SOCK_MACRAW :
					if(socket(sn, Sn_MR_MACRAW, dummyport, 0x80) == sn) // Flag: [0x80] MAC Filter Enabled
					{
#ifdef _TEST_DEBUG_
						//printf("%d:Socket Open: MACRAW mode\r\n",sn);
#endif
					}
					break;

				case SOCK_UDP :
					dummyport = 5000;
					if(socket(sn, Sn_MR_UDP, dummyport, 0) == sn) // Flag: [0x80] MAC Filter Enabled
					{
#ifdef _TEST_DEBUG_
						//printf("%d:Socket Open: UDP mode\r\n",sn);
#endif
					}
					break;

				default:
					break;
			}
			break;

		default :
			break;
	}
	return len;
}

int ADC_Read(ADC_CH num)
{
	ADC_ChannelSelect (num); ///< Select ADC channel to CH0
	ADC_Start(); ///< Start ADC

	while(ADC_IsEOC()); ///< Wait until End of Conversion

	return ((uint16_t)ADC_ReadData()); ///< read ADC Data
}
